unit WicUnit;

interface

uses Vcl.Graphics, System.Classes, Winapi.wincodec, Winapi.ActiveX;

Type TWICImage = Class (Vcl.Graphics.TWICImage)
public
  procedure SaveToFile(const Filename: string); override;
  procedure SaveToStream(Stream: TStream); override;
end;

implementation

procedure TWICImage.SaveToFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TWICImage.SaveToStream(Stream: TStream);
var
  FGData: TMemoryStream;
  Encoder: IWICBitmapEncoder;
  Frame: IWICBitmapFrameEncode;
  Props: IPropertyBag2;
  LStreamAdapter: IStream;
  PixelFormat: TGUID;
  LStream: IWICStream;
  Palette: IWICPalette;
begin
  FGData := TMemoryStream.Create;
  try
    LStreamAdapter := TStreamAdapter.Create(FGData);

    ImagingFactory.CreateStream(LStream);
    LStream.InitializeFromIStream(LStreamAdapter);
    ImagingFactory.CreateEncoder(EncoderContainerFormat, guid_null, Encoder);

    Encoder.Initialize(LStream, WICBitmapEncoderNoCache);
    Encoder.CreateNewFrame(Frame, Props);

    Frame.Initialize(Props);
    Handle.GetPixelFormat(PixelFormat);
    Frame.SetPixelFormat(PixelFormat);

    Frame.SetSize(Width, Height);

    ImagingFactory.CreatePalette(Palette);
    Handle.CopyPalette(Palette);
    Frame.SetPalette(Palette);
    Frame.WriteSource(Handle, nil);
    Frame.Commit;
    Encoder.Commit;

    FGData.Position := 0;
    Stream.CopyFrom(FGData, FGData.Size);
  finally
    FGData.Free;
  end;
end;


end.
